from typing import Optional
from uuid import UUID

from pydantic import BaseModel

from lqs.interface.core.models.__common__ import (
    CommonModel,
    DataResponseModel,
    PaginationModel,
    ProcessState,
    ProcessType,
    optional_field,
)


class Hook(CommonModel):
    workflow_id: UUID
    trigger_process: ProcessType
    trigger_state: ProcessState
    name: Optional[str]
    note: Optional[str]
    context: Optional[dict]
    managed: bool
    disabled: bool
    uri: Optional[str]


class HookDataResponse(DataResponseModel[Hook]):
    pass


class HookListResponse(PaginationModel[Hook]):
    pass


class HookCreateRequest(BaseModel):
    trigger_process: ProcessType
    trigger_state: ProcessState
    name: Optional[str] = None
    note: Optional[str] = None
    context: Optional[dict] = None
    disabled: bool = False
    managed: bool = False
    uri: Optional[str] = None
    secret: Optional[str] = None


class HookUpdateRequest(BaseModel):
    trigger_process: ProcessType = optional_field
    trigger_state: ProcessState = optional_field
    name: Optional[str] = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    disabled: bool = optional_field
    uri: Optional[str] = optional_field
    secret: Optional[str] = optional_field



from typing import Optional, List, Union
from datetime import datetime
from uuid import UUID
from abc import abstractmethod

import lqs.interface.core.models as models
from lqs.interface.base.list import ListInterface as BaseListInterface


class ListInterface(BaseListInterface):
    @abstractmethod
    def _digestion(self, **kwargs) -> models.DigestionListResponse:
        pass

    def digestion(
        self,
        id: Optional[UUID] = None,
        group_id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        workflow_id: Optional[UUID] = None,
        workflow_id_null: Optional[bool] = None,
        state: Optional[str] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        progress_null: Optional[bool] = None,
        progress_gte: Optional[float] = None,
        progress_lte: Optional[float] = None,
        error_filter: Optional[str] = None,
        note_like: Optional[str] = None,
        context_filter: Optional[str] = None,
        workflow_context_filter: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists digestions.
        """
        return self._digestion(
            id=id,
            group_id=group_id,
            log_id=log_id,
            workflow_id=workflow_id,
            workflow_id_null=workflow_id_null,
            state=state,
            name=name,
            name_like=name_like,
            progress_null=progress_null,
            progress_gte=progress_gte,
            progress_lte=progress_lte,
            error_filter=error_filter,
            note_like=note_like,
            context_filter=context_filter,
            workflow_context_filter=workflow_context_filter,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def digestions(self, **kwargs):
        return self.digestion(**kwargs)

    @abstractmethod
    def _digestion_part(self, **kwargs) -> models.DigestionPartListResponse:
        pass

    def digestion_part(
        self,
        id: Optional[UUID] = None,
        group_id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        sequence: Optional[int] = None,
        digestion_id: Optional[UUID] = None,
        workflow_id: Optional[UUID] = None,
        workflow_id_null: Optional[bool] = None,
        state: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists digestion parts.
        """
        return self._digestion_part(
            id=id,
            group_id=group_id,
            log_id=log_id,
            sequence=sequence,
            digestion_id=digestion_id,
            workflow_id=workflow_id,
            workflow_id_null=workflow_id_null,
            state=state,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def digestion_parts(self, **kwargs):
        return self.digestion_part(**kwargs)

    @abstractmethod
    def _digestion_topic(self, **kwargs) -> models.DigestionTopicListResponse:
        pass

    def digestion_topic(
        self,
        id: Optional[UUID] = None,
        digestion_id: Optional[UUID] = None,
        group_id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        topic_id: Optional[UUID] = None,
        start_time_null: Optional[bool] = None,
        start_time_gte: Optional[models.Int64] = None,
        start_time_lte: Optional[models.Int64] = None,
        end_time_null: Optional[bool] = None,
        end_time_gte: Optional[models.Int64] = None,
        end_time_lte: Optional[models.Int64] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists digestion topics.
        """
        return self._digestion_topic(
            id=id,
            group_id=group_id,
            log_id=log_id,
            digestion_id=digestion_id,
            topic_id=topic_id,
            start_time_null=start_time_null,
            start_time_gte=start_time_gte,
            start_time_lte=start_time_lte,
            end_time_null=end_time_null,
            end_time_gte=end_time_gte,
            end_time_lte=end_time_lte,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def digestion_topics(self, **kwargs):
        return self.digestion_topic(**kwargs)

    @abstractmethod
    def _group(self, **kwargs) -> models.GroupListResponse:
        pass

    def group(
        self,
        id: Optional[UUID] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        default_workflow_id: Optional[UUID] = None,
        default_workflow_id_null: Optional[bool] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists groups.
        """
        return self._group(
            id=id,
            name=name,
            name_like=name_like,
            default_workflow_id=default_workflow_id,
            default_workflow_id_null=default_workflow_id_null,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def groups(self, **kwargs):
        return self.group(**kwargs)

    @abstractmethod
    def _hook(self, **kwargs) -> models.HookListResponse:
        pass

    def hook(
        self,
        id: Optional[UUID] = None,
        workflow_id: Optional[UUID] = None,
        trigger_process: Optional[str] = None,
        trigger_state: Optional[str] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        note_like: Optional[str] = None,
        managed: Optional[bool] = None,
        disabled: Optional[bool] = None,
        uri: Optional[str] = None,
        uri_like: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists hooks.
        """
        return self._hook(
            id=id,
            workflow_id=workflow_id,
            trigger_process=trigger_process,
            trigger_state=trigger_state,
            name=name,
            name_like=name_like,
            note_like=note_like,
            uri=uri,
            uri_like=uri_like,
            managed=managed,
            disabled=disabled,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def hooks(self, **kwargs):
        return self.hook(**kwargs)

    @abstractmethod
    def _ingestion(self, **kwargs) -> models.IngestionListResponse:
        pass

    def ingestion(
        self,
        id: Optional[UUID] = None,
        group_id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        object_store_id: Optional[UUID] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        object_key: Optional[str] = None,
        object_key_like: Optional[str] = None,
        workflow_id: Optional[UUID] = None,
        workflow_id_null: Optional[bool] = None,
        workflow_context_filter: Optional[str] = None,
        state: Optional[str] = None,
        progress_null: Optional[bool] = None,
        progress_gte: Optional[float] = None,
        progress_lte: Optional[float] = None,
        error_filter: Optional[str] = None,
        note_like: Optional[str] = None,
        context_filter: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists ingestions.
        """
        return self._ingestion(
            id=id,
            group_id=group_id,
            log_id=log_id,
            object_store_id=object_store_id,
            name=name,
            name_like=name_like,
            object_key=object_key,
            object_key_like=object_key_like,
            workflow_id=workflow_id,
            workflow_id_null=workflow_id_null,
            workflow_context_filter=workflow_context_filter,
            state=state,
            progress_null=progress_null,
            progress_gte=progress_gte,
            progress_lte=progress_lte,
            error_filter=error_filter,
            note_like=note_like,
            context_filter=context_filter,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def ingestions(self, **kwargs):
        return self.ingestion(**kwargs)

    @abstractmethod
    def _ingestion_part(self, **kwargs) -> models.IngestionPartListResponse:
        pass

    def ingestion_part(
        self,
        id: Optional[UUID] = None,
        group_id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        ingestion_id: Optional[UUID] = None,
        sequence: Optional[int] = None,
        source: Optional[str] = None,
        workflow_id: Optional[UUID] = None,
        workflow_id_null: Optional[bool] = None,
        state: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists ingestion parts.
        """
        return self._ingestion_part(
            id=id,
            group_id=group_id,
            log_id=log_id,
            sequence=sequence,
            ingestion_id=ingestion_id,
            workflow_id=workflow_id,
            workflow_id_null=workflow_id_null,
            state=state,
            source=source,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def ingestion_parts(self, **kwargs):
        return self.ingestion_part(**kwargs)

    @abstractmethod
    def _label(self, **kwargs) -> models.LabelListResponse:
        pass

    def label(
        self,
        id: Optional[UUID] = None,
        value: Optional[str] = None,
        value_like: Optional[str] = None,
        note_like: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists labels.
        """
        return self._label(
            id=id,
            value=value,
            value_like=value_like,
            note_like=note_like,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def labels(self, **kwargs):
        return self.label(**kwargs)

    @abstractmethod
    def _log(self, **kwargs) -> models.LogListResponse:
        pass

    def log(
        self,
        id: Optional[UUID] = None,
        group_id: Optional[UUID] = None,
        default_workflow_id: Optional[UUID] = None,
        default_workflow_id_null: Optional[bool] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        locked: Optional[bool] = None,
        note_like: Optional[str] = None,
        context_filter: Optional[str] = None,
        start_time_null: Optional[bool] = None,
        start_time_gte: Optional[models.Int64] = None,
        start_time_lte: Optional[models.Int64] = None,
        end_time_null: Optional[bool] = None,
        end_time_gte: Optional[models.Int64] = None,
        end_time_lte: Optional[models.Int64] = None,
        record_size_gte: Optional[int] = None,
        record_size_lte: Optional[int] = None,
        record_count_gte: Optional[int] = None,
        record_count_lte: Optional[int] = None,
        object_size_gte: Optional[int] = None,
        object_size_lte: Optional[int] = None,
        object_count_gte: Optional[int] = None,
        object_count_lte: Optional[int] = None,
        tag_label_ids_includes: Optional[Union[UUID, List[UUID]]] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists logs.
        """
        return self._log(
            id=id,
            group_id=group_id,
            default_workflow_id=default_workflow_id,
            default_workflow_id_null=default_workflow_id_null,
            name=name,
            name_like=name_like,
            locked=locked,
            note_like=note_like,
            context_filter=context_filter,
            start_time_null=start_time_null,
            start_time_gte=start_time_gte,
            start_time_lte=start_time_lte,
            end_time_null=end_time_null,
            end_time_gte=end_time_gte,
            end_time_lte=end_time_lte,
            record_size_gte=record_size_gte,
            record_size_lte=record_size_lte,
            record_count_gte=record_count_gte,
            record_count_lte=record_count_lte,
            object_size_gte=object_size_gte,
            object_size_lte=object_size_lte,
            object_count_gte=object_count_gte,
            object_count_lte=object_count_lte,
            tag_label_ids_includes=tag_label_ids_includes,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def logs(self, **kwargs):
        return self.log(**kwargs)

    @abstractmethod
    def _log_object(self, **kwargs) -> models.ObjectListResponse:
        pass

    def log_object(
        self,
        log_id: UUID,
        processing: Optional[bool] = False,
        max_keys: Optional[int] = None,
        prefix: Optional[str] = None,
        start_after: Optional[str] = None,
        delimiter: Optional[str] = None,
        continuation_token: Optional[str] = None,
    ):
        """
        Lists log objects.
        """
        return self._log_object(
            log_id=log_id,
            processing=processing,
            max_keys=max_keys,
            prefix=prefix,
            start_after=start_after,
            delimiter=delimiter,
            continuation_token=continuation_token,
        )

    def log_objects(self, **kwargs):
        return self.log_object(**kwargs)

    @abstractmethod
    def _log_object_part(self, **kwargs) -> models.ObjectPartListResponse:
        pass

    def log_object_part(
        self,
        object_key: str,
        log_id: UUID,
        max_parts: Optional[int] = None,
        part_number_marker: Optional[int] = None,
    ):
        """
        Lists log object parts.
        """
        return self._log_object_part(
            log_id=log_id,
            object_key=object_key,
            max_parts=max_parts,
            part_number_marker=part_number_marker,
        )

    def log_object_parts(self, **kwargs):
        return self.log_object_part(**kwargs)

    @abstractmethod
    def _object(self, **kwargs) -> models.ObjectListResponse:
        pass

    def object(
        self,
        object_store_id: UUID,
        processing: Optional[bool] = False,
        max_keys: Optional[int] = None,
        prefix: Optional[str] = None,
        start_after: Optional[str] = None,
        delimiter: Optional[str] = None,
        continuation_token: Optional[str] = None,
    ):
        """
        Lists objects.
        """
        return self._object(
            object_store_id=object_store_id,
            processing=processing,
            max_keys=max_keys,
            prefix=prefix,
            start_after=start_after,
            delimiter=delimiter,
            continuation_token=continuation_token,
        )

    def objects(self, **kwargs):
        return self.object(**kwargs)

    @abstractmethod
    def _object_part(self, **kwargs) -> models.ObjectPartListResponse:
        pass

    def object_part(
        self,
        object_key: str,
        object_store_id: UUID,
        max_parts: Optional[int] = None,
        part_number_marker: Optional[int] = None,
    ):
        """
        Lists object parts.
        """
        return self._object_part(
            object_store_id=object_store_id,
            object_key=object_key,
            max_parts=max_parts,
            part_number_marker=part_number_marker,
        )

    def object_parts(self, **kwargs):
        return self.object_part(**kwargs)

    @abstractmethod
    def _object_store(self, **kwargs) -> models.ObjectStoreListResponse:
        pass

    def object_store(
        self,
        id: Optional[UUID] = None,
        bucket_name: Optional[str] = None,
        bucket_name_like: Optional[str] = None,
        access_key_id: Optional[str] = None,
        region_name: Optional[str] = None,
        endpoint_url: Optional[str] = None,
        note: Optional[str] = None,
        note_like: Optional[str] = None,
        disabled: Optional[bool] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists object stores.
        """
        return self._object_store(
            id=id,
            bucket_name=bucket_name,
            bucket_name_like=bucket_name_like,
            access_key_id=access_key_id,
            region_name=region_name,
            endpoint_url=endpoint_url,
            note=note,
            note_like=note_like,
            disabled=disabled,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def object_stores(self, **kwargs):
        return self.object_store(**kwargs)

    @abstractmethod
    def _query(self, **kwargs) -> models.QueryListResponse:
        pass

    def query(
        self,
        id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        note_like: Optional[str] = None,
        statement: Optional[str] = None,
        statement_like: Optional[str] = None,
        workflow_id: Optional[UUID] = None,
        workflow_id_null: Optional[bool] = None,
        workflow_context_filter: Optional[str] = None,
        context_filter: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists queries.
        """
        return self._query(
            id=id,
            log_id=log_id,
            name=name,
            name_like=name_like,
            note_like=note_like,
            statement=statement,
            statement_like=statement_like,
            workflow_id=workflow_id,
            workflow_id_null=workflow_id_null,
            workflow_context_filter=workflow_context_filter,
            context_filter=context_filter,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def queries(self, **kwargs):
        return self.query(**kwargs)

    @abstractmethod
    def _record(self, **kwargs) -> models.RecordListResponse:
        pass

    def record(
        self,
        topic_id: UUID,
        timestamp: Optional[models.Int64] = None,
        log_id: Optional[UUID] = None,
        ingestion_id: Optional[UUID] = None,
        workflow_id: Optional[UUID] = None,
        workflow_id_null: Optional[bool] = None,
        error_filter: Optional[str] = None,
        note_like: Optional[str] = None,
        source: Optional[str] = None,
        query_data_filter: Optional[str] = None,
        context_filter: Optional[str] = None,
        altered: Optional[bool] = None,
        frequency: Optional[float] = None,
        timestamp_gt: Optional[models.Int64] = None,
        timestamp_lt: Optional[models.Int64] = None,
        timestamp_gte: Optional[models.Int64] = None,
        timestamp_lte: Optional[models.Int64] = None,
        data_length_gte: Optional[int] = None,
        data_length_lte: Optional[int] = None,
        data_offset_gte: Optional[int] = None,
        data_offset_lte: Optional[int] = None,
        chunk_compression: Optional[str] = None,
        chunk_offset_gte: Optional[int] = None,
        chunk_offset_lte: Optional[int] = None,
        chunk_length_gte: Optional[int] = None,
        chunk_length_lte: Optional[int] = None,
        include_auxiliary_data: Optional[bool] = False,
        include_raw_data: Optional[bool] = False,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "timestamp",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists records.
        """
        return self._record(
            topic_id=topic_id,
            timestamp=timestamp,
            log_id=log_id,
            ingestion_id=ingestion_id,
            workflow_id=workflow_id,
            workflow_id_null=workflow_id_null,
            error_filter=error_filter,
            note_like=note_like,
            source=source,
            query_data_filter=query_data_filter,
            context_filter=context_filter,
            altered=altered,
            frequency=frequency,
            timestamp_gt=timestamp_gt,
            timestamp_lt=timestamp_lt,
            timestamp_gte=timestamp_gte,
            timestamp_lte=timestamp_lte,
            data_length_gte=data_length_gte,
            data_length_lte=data_length_lte,
            data_offset_gte=data_offset_gte,
            data_offset_lte=data_offset_lte,
            chunk_compression=chunk_compression,
            chunk_offset_gte=chunk_offset_gte,
            chunk_offset_lte=chunk_offset_lte,
            chunk_length_gte=chunk_length_gte,
            chunk_length_lte=chunk_length_lte,
            include_auxiliary_data=include_auxiliary_data,
            include_raw_data=include_raw_data,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def records(self, **kwargs):
        return self.record(**kwargs)

    @abstractmethod
    def _tag(self, **kwargs) -> models.TagListResponse:
        pass

    def tag(
        self,
        id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        label_id: Optional[UUID] = None,
        topic_id: Optional[UUID] = None,
        note: Optional[str] = None,
        note_like: Optional[str] = None,
        context_filter: Optional[str] = None,
        start_time_null: Optional[bool] = None,
        start_time_gte: Optional[models.Int64] = None,
        start_time_lte: Optional[models.Int64] = None,
        end_time_null: Optional[bool] = None,
        end_time_gte: Optional[models.Int64] = None,
        end_time_lte: Optional[models.Int64] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists tags.
        """
        return self._tag(
            id=id,
            label_id=label_id,
            log_id=log_id,
            topic_id=topic_id,
            note=note,
            note_like=note_like,
            context_filter=context_filter,
            start_time_null=start_time_null,
            start_time_gte=start_time_gte,
            start_time_lte=start_time_lte,
            end_time_null=end_time_null,
            end_time_gte=end_time_gte,
            end_time_lte=end_time_lte,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def tags(self, **kwargs):
        return self.tag(**kwargs)

    @abstractmethod
    def _topic(self, **kwargs) -> models.TopicListResponse:
        pass

    def topic(
        self,
        id: Optional[UUID] = None,
        log_id: Optional[UUID] = None,
        group_id: Optional[UUID] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        associated_topic_id: Optional[UUID] = None,
        latched: Optional[bool] = None,
        strict: Optional[bool] = None,
        locked: Optional[bool] = None,
        context_filter: Optional[str] = None,
        start_time_null: Optional[bool] = None,
        start_time_gte: Optional[models.Int64] = None,
        start_time_lte: Optional[models.Int64] = None,
        end_time_null: Optional[bool] = None,
        end_time_gte: Optional[models.Int64] = None,
        end_time_lte: Optional[models.Int64] = None,
        record_size_gte: Optional[int] = None,
        record_size_lte: Optional[int] = None,
        record_count_gte: Optional[int] = None,
        record_count_lte: Optional[int] = None,
        type_name: Optional[str] = None,
        type_name_like: Optional[str] = None,
        type_encoding: Optional[models.TypeEncoding] = None,
        type_data: Optional[str] = None,
        type_data_like: Optional[str] = None,
        type_schema_filter: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists topics.
        """
        return self._topic(
            id=id,
            group_id=group_id,
            name=name,
            name_like=name_like,
            log_id=log_id,
            associated_topic_id=associated_topic_id,
            latched=latched,
            strict=strict,
            locked=locked,
            context_filter=context_filter,
            start_time_null=start_time_null,
            start_time_gte=start_time_gte,
            start_time_lte=start_time_lte,
            end_time_null=end_time_null,
            end_time_gte=end_time_gte,
            end_time_lte=end_time_lte,
            record_size_gte=record_size_gte,
            record_size_lte=record_size_lte,
            record_count_gte=record_count_gte,
            record_count_lte=record_count_lte,
            type_name=type_name,
            type_name_like=type_name_like,
            type_encoding=type_encoding,
            type_data=type_data,
            type_data_like=type_data_like,
            type_schema_filter=type_schema_filter,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def topics(self, **kwargs):
        return self.topic(**kwargs)

    @abstractmethod
    def _workflow(self, **kwargs) -> models.WorkflowListResponse:
        pass

    def workflow(
        self,
        id: Optional[UUID] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        default: Optional[bool] = None,
        disabled: Optional[bool] = None,
        managed: Optional[bool] = None,
        context_schema_filter: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        """
        Lists workflows.
        """
        return self._workflow(
            id=id,
            name=name,
            name_like=name_like,
            default=default,
            disabled=disabled,
            managed=managed,
            context_schema_filter=context_schema_filter,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def workflows(self, **kwargs):
        return self.workflow(**kwargs)



from lqs.interface.core import UpdateInterface
from lqs.client.common import RESTInterface
import lqs.interface.core.models as models


class Update(UpdateInterface, RESTInterface):
    service: str = "lqs"

    def __init__(self, app):
        super().__init__(app=app)

    def _api_key(self, **params):
        api_key_id = params.pop("api_key_id")
        data = params.pop("data")
        return self._update_resource(
            f"apiKeys/{api_key_id}", data, models.APIKeyDataResponse
        )

    def _digestion(self, **params):
        digestion_id = params.pop("digestion_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"digestions/{digestion_id}",
            data,
            models.DigestionDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _digestion_part(self, **params):
        digestion_id = params.pop("digestion_id")
        digestion_part_id = params.pop("digestion_part_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"digestions/{digestion_id}/parts/{digestion_part_id}",
            data,
            models.DigestionPartDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _digestion_topic(self, **params):
        digestion_id = params.pop("digestion_id")
        digestion_topic_id = params.pop("digestion_topic_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"digestions/{digestion_id}/topics/{digestion_topic_id}",
            data,
            models.DigestionTopicDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _group(self, **params):
        group_id = params.pop("group_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"groups/{group_id}",
            data,
            models.GroupDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _hook(self, **params):
        workflow_id = params.pop("workflow_id")
        hook_id = params.pop("hook_id")
        data = params.pop("data")
        return self._update_resource(
            f"workflows/{workflow_id}/hooks/{hook_id}", data, models.HookDataResponse
        )

    def _ingestion(self, **params):
        ingestion_id = params.pop("ingestion_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"ingestions/{ingestion_id}",
            data,
            models.IngestionDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _ingestion_part(self, **params):
        ingestion_id = params.pop("ingestion_id")
        ingestion_part_id = params.pop("ingestion_part_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"ingestions/{ingestion_id}/parts/{ingestion_part_id}",
            data,
            models.IngestionPartDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _label(self, **params):
        label_id = params.pop("label_id")
        data = params.pop("data")
        return self._update_resource(
            f"labels/{label_id}", data, models.LabelDataResponse
        )

    def _log(self, **params):
        log_id = params.pop("log_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"logs/{log_id}",
            data,
            models.LogDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _log_object(self, **params):
        log_id = params.pop("log_id")
        object_key = params.pop("object_key")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"logs/{log_id}/objects/{object_key}",
            data,
            models.ObjectDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _object(self, **params):
        raise NotImplementedError

    def _object_store(self, **params):
        object_store_id = params.pop("object_store_id")
        data = params.pop("data")
        return self._update_resource(
            f"objectStores/{object_store_id}", data, models.ObjectStoreDataResponse
        )

    def _query(self, **params):
        log_id = params.pop("log_id")
        query_id = params.pop("query_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"logs/{log_id}/queries/{query_id}",
            data,
            models.QueryDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _record(self, **params):
        topic_id = params.pop("topic_id")
        timestamp = params.pop("timestamp")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"topics/{topic_id}/records/{timestamp}",
            data,
            models.RecordDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _role(self, **params):
        role_id = params.pop("role_id")
        data = params.pop("data")
        return self._update_resource(f"roles/{role_id}", data, models.RoleDataResponse)

    def _tag(self, **params):
        log_id = params.pop("log_id")
        tag_id = params.pop("tag_id")
        lock_token = params.pop("lock_token", None)
        data = params.pop("data")
        return self._update_resource(
            f"logs/{log_id}/tags/{tag_id}",
            data,
            models.TagDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _topic(self, **params):
        topic_id = params.pop("topic_id")
        data = params.pop("data")
        lock_token = params.pop("lock_token", None)
        return self._update_resource(
            f"topics/{topic_id}",
            data,
            models.TopicDataResponse,
            additiona_params={"lock_token": lock_token},
        )

    def _user(self, **params):
        user_id = params.pop("user_id")
        data = params.pop("data")
        return self._update_resource(f"users/{user_id}", data, models.UserDataResponse)

    def _workflow(self, **params):
        workflow_id = params.pop("workflow_id")
        data = params.pop("data")
        return self._update_resource(
            f"workflows/{workflow_id}", data, models.WorkflowDataResponse
        )



from typing import Optional

from pydantic import BaseModel

from lqs.interface.base.models.__common__ import (
    CommonModel,
    DataResponseModel,
    PaginationModel,
    optional_field,
)


class ObjectStore(CommonModel):
    bucket_name: str
    access_key_id: Optional[str]
    region_name: Optional[str]
    endpoint_url: Optional[str]
    note: Optional[str]
    context: Optional[dict]
    disabled: bool


class ObjectStoreDataResponse(DataResponseModel[ObjectStore]):
    pass


class ObjectStoreListResponse(PaginationModel[ObjectStore]):
    pass


class ObjectStoreCreateRequest(BaseModel):
    bucket_name: str
    access_key_id: Optional[str] = None
    secret_access_key: Optional[str] = None
    region_name: Optional[str] = None
    endpoint_url: Optional[str] = None
    note: Optional[str] = None
    context: Optional[dict] = None
    disabled: bool = False


class ObjectStoreUpdateRequest(BaseModel):
    bucket_name: str = optional_field
    access_key_id: Optional[str] = optional_field
    secret_access_key: Optional[str] = optional_field
    region_name: Optional[str] = optional_field
    endpoint_url: Optional[str] = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    disabled: bool = optional_field



from typing import Optional
from uuid import UUID

from pydantic import BaseModel

from lqs.interface.dsm.models.__common__ import (
    CommonModel,
    DataResponseModel,
    PaginationModel,
    optional_field,
)


class Comment(CommonModel):
    user_id: Optional[UUID]
    datastore_id: Optional[UUID]
    resource_type: Optional[str]
    resource_id: Optional[UUID]

    subject: Optional[str]
    content: Optional[str]
    context: Optional[dict]
    status: Optional[str]


class CommentDataResponse(DataResponseModel[Comment]):
    pass


class CommentListResponse(PaginationModel[Comment]):
    pass


class CommentCreateRequest(BaseModel):
    user_id: Optional[UUID] = None
    datastore_id: Optional[UUID] = None
    resource_type: Optional[str] = None
    resource_id: Optional[UUID] = None

    subject: Optional[str] = None
    content: Optional[str] = None
    context: Optional[dict] = None


class CommentUpdateRequest(BaseModel):
    user_id: Optional[UUID] = optional_field
    datastore_id: Optional[UUID] = optional_field
    resource_type: Optional[str] = optional_field
    resource_id: Optional[UUID] = optional_field

    subject: Optional[str] = optional_field
    content: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    status: Optional[str] = optional_field



from typing import List, Optional, Dict, Any
from uuid import UUID

from pydantic import BaseModel

from lqs.interface.core.models.__common__ import (
    CommonModel,
    DataResponseModel,
    PaginationModel,
    optional_field,
)


class Query(CommonModel):
    log_id: UUID
    name: Optional[str]
    note: Optional[str]
    context: Optional[dict]

    statement: Optional[str]
    parameters: Optional[Dict[str, Any]]
    columns: Optional[List[str]]
    rows: Optional[List[List[Any]]]
    error: Optional[dict]


class QueryDataResponse(DataResponseModel[Query]):
    pass


class QueryListResponse(PaginationModel[Query]):
    pass


class QueryCreateRequest(BaseModel):
    name: Optional[str] = None
    note: Optional[str] = None
    context: Optional[dict] = None
    statement: Optional[str] = None
    parameters: Optional[Dict[str, Any]] = None


class QueryUpdateRequest(BaseModel):
    name: Optional[str] = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    statement: Optional[str] = optional_field
    parameters: Optional[Dict[str, Any]] = optional_field



from typing import Optional
from datetime import datetime
from uuid import UUID
from abc import ABC, abstractmethod

import lqs.interface.base.models as models


class ListInterface(ABC):
    @abstractmethod
    def _api_key(self, **kwargs) -> models.APIKeyListResponse:
        pass

    def api_key(
        self,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        user_id: Optional[UUID] = None,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ) -> models.APIKeyListResponse:
        return self._api_key(
            name=name,
            name_like=name_like,
            user_id=user_id,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def api_keys(self, **kwargs) -> models.APIKeyListResponse:
        return self.api_key(**kwargs)

    @abstractmethod
    def _role(self, **kwargs) -> models.RoleListResponse:
        pass

    def role(
        self,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        disabled: Optional[bool] = None,
        default: Optional[bool] = None,
        managed: Optional[bool] = None,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ) -> models.RoleListResponse:
        return self._role(
            name=name,
            name_like=name_like,
            disabled=disabled,
            default=default,
            managed=managed,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def roles(self, **kwargs) -> models.RoleListResponse:
        return self.role(**kwargs)

    @abstractmethod
    def _user(self, **kwargs) -> models.UserListResponse:
        pass

    def user(
        self,
        username: Optional[str] = None,
        username_like: Optional[str] = None,
        role_id: Optional[UUID] = None,
        managed: Optional[bool] = None,
        disabled: Optional[bool] = None,
        admin: Optional[bool] = None,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ) -> models.UserListResponse:
        return self._user(
            username=username,
            username_like=username_like,
            role_id=role_id,
            managed=managed,
            disabled=disabled,
            admin=admin,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def users(self, **kwargs) -> models.UserListResponse:
        return self.user(**kwargs)



from lqs.interface.dsm import CreateInterface
from lqs.client.common import RESTInterface
import lqs.interface.dsm.models as models


class Create(CreateInterface, RESTInterface):
    service: str = "dsm"

    def __init__(self, app):
        super().__init__(app=app)

    def _announcement(self, **params):
        return self._create_resource(
            "announcements", params, models.AnnouncementDataResponse
        )

    def _api_key(self, **params):
        return self._create_resource("apiKeys", params, models.APIKeyDataResponse)

    def _comment(self, **params):
        return self._create_resource("comments", params, models.CommentDataResponse)

    def _configuration(self, **params):
        return self._create_resource(
            "configurations", params, models.ConfigurationDataResponse
        )

    def _datastore(self, **params):
        return self._create_resource("dataStores", params, models.DataStoreDataResponse)

    def _datastore_association(self, **params):
        return self._create_resource(
            "dataStoreAssociations", params, models.DataStoreAssociationDataResponse
        )

    def _event(self, **params):
        return self._create_resource("events", params, models.EventDataResponse)

    def _inference(self, **params):
        return self._create_resource("inferences", params, models.InferenceDataResponse)

    def _job(self, **params):
        return self._create_resource("jobs", params, models.JobDataResponse)

    def _role(self, **params):
        return self._create_resource("roles", params, models.RoleDataResponse)

    def _ticket(self, **params):
        return self._create_resource("tickets", params, models.TicketDataResponse)

    def _usage_record(self, **params):
        return self._create_resource(
            "usageRecords",
            params,
            models.UsageRecordDataResponse,
        )

    def _user(self, **params):
        return self._create_resource("users", params, models.UserDataResponse)



from abc import ABC, abstractmethod
from uuid import UUID


class DeleteInterface(ABC):
    @abstractmethod
    def _api_key(self, **kwargs):
        pass

    def api_key(self, api_key_id: UUID):
        return self._api_key(
            api_key_id=api_key_id,
        )

    @abstractmethod
    def _role(self, **kwargs):
        pass

    def role(self, role_id: UUID):
        return self._role(
            role_id=role_id,
        )

    @abstractmethod
    def _user(self, **kwargs):
        pass

    def user(self, user_id: UUID):
        return self._user(
            user_id=user_id,
        )



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from lqs.client import RESTClient
from lqs.client.core.list import List
from lqs.client.core.fetch import Fetch
from lqs.client.core.create import Create
from lqs.client.core.update import Update
from lqs.client.core.delete import Delete
from lqs.interface.core.models import Resources


class LogQS:
    def __init__(self, app: "RESTClient"):
        self.app = app

        self.resource = Resources
        for resource_name in dir(self.resource):
            if resource_name.startswith("_"):
                continue
            resource = getattr(self.resource, resource_name)
            try:
                resource.set_app(self.app)
            except AttributeError:
                self.app.logger.debug(f"Failed to set app for {resource_name}.")

        self.list = List(app=app)
        self.fetch = Fetch(app=app)
        self.create = Create(app=app)
        self.update = Update(app=app)
        self.delete = Delete(app=app)



from uuid import UUID
from typing import Optional

from lqs.interface.core import DeleteInterface
from lqs.client.common import RESTInterface

# TODO: make this consistent with other interfaces


class Delete(DeleteInterface, RESTInterface):
    service: str = "lqs"

    def __init__(self, app):
        super().__init__(app=app)

    def _api_key(self, api_key_id: UUID):
        self._delete_resource(f"apiKeys/{api_key_id}")
        return

    def _digestion(self, digestion_id: UUID, lock_token: Optional[str] = None):
        self._delete_resource(
            f"digestions/{digestion_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _digestion_part(
        self,
        digestion_id: UUID,
        digestion_part_id: UUID,
        lock_token: Optional[str] = None,
    ):
        self._delete_resource(
            f"digestions/{digestion_id}/parts/{digestion_part_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _digestion_topic(
        self,
        digestion_id: UUID,
        digestion_topic_id: UUID,
        lock_token: Optional[str] = None,
    ):
        self._delete_resource(
            f"digestions/{digestion_id}/topics/{digestion_topic_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _group(self, group_id: UUID, lock_token: Optional[str] = None):
        self._delete_resource(
            f"groups/{group_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _hook(self, workflow_id: UUID, hook_id: UUID):
        self._delete_resource(f"workflows/{workflow_id}/hooks/{hook_id}")
        return

    def _ingestion(self, ingestion_id: UUID, lock_token: Optional[str] = None):
        self._delete_resource(
            f"ingestions/{ingestion_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _ingestion_part(
        self,
        ingestion_id: UUID,
        ingestion_part_id: UUID,
        lock_token: Optional[str] = None,
    ):
        self._delete_resource(
            f"ingestions/{ingestion_id}/parts/{ingestion_part_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _label(self, label_id: UUID):
        self._delete_resource(f"labels/{label_id}")
        return

    def _log(self, log_id: UUID, lock_token: Optional[str] = None):
        self._delete_resource(
            f"logs/{log_id}", additiona_params={"lock_token": lock_token}
        )
        return

    def _log_object(
        self, log_id: UUID, object_key: str, lock_token: Optional[str] = None
    ):
        self._delete_resource(
            f"logs/{log_id}/objects/{object_key}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _object(self, object_store_id: UUID, object_key: str):
        raise NotImplementedError

    def _object_store(self, object_store_id: UUID):
        self._delete_resource(f"objectStores/{object_store_id}")
        return

    def _query(self, log_id: UUID, query_id: UUID, lock_token: Optional[str] = None):
        self._delete_resource(
            f"logs/{log_id}/queries/{query_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _record(
        self, timestamp: float, topic_id: UUID, lock_token: Optional[str] = None
    ):
        self._delete_resource(
            f"topics/{topic_id}/records/{timestamp}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _role(self, role_id: UUID):
        self._delete_resource(f"roles/{role_id}")
        return

    def _tag(self, log_id: UUID, tag_id: UUID, lock_token: Optional[str] = None):
        self._delete_resource(
            f"logs/{log_id}/tags/{tag_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _topic(self, topic_id: UUID, lock_token: Optional[str] = None):
        self._delete_resource(
            f"topics/{topic_id}",
            additiona_params={"lock_token": lock_token},
        )
        return

    def _user(self, user_id: UUID):
        self._delete_resource(f"users/{user_id}")
        return

    def _workflow(self, workflow_id: UUID):
        self._delete_resource(f"workflows/{workflow_id}")
        return



from lqs.interface.dsm import ListInterface
from lqs.client.common import RESTInterface
import lqs.interface.dsm.models as models


class List(ListInterface, RESTInterface):
    service: str = "dsm"

    def __init__(self, app):
        super().__init__(app=app)

    def _announcement(self, **params):
        resource_path = "announcements" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.AnnouncementListResponse
        )
        return result

    def _api_key(self, **params):
        resource_path = "apiKeys" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.APIKeyListResponse
        )
        return result

    def _comment(self, **params):
        resource_path = "comments" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.CommentListResponse
        )
        return result

    def _configuration(self, **params):
        resource_path = "configurations" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.ConfigurationListResponse
        )
        return result

    def _datastore(self, **params):
        resource_path = "dataStores" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.DataStoreListResponse
        )
        return result

    def _datastore_association(self, **params):
        resource_path = "dataStoreAssociations" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.DataStoreAssociationListResponse
        )
        return result

    def _event(self, **params):
        resource_path = "events" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.EventListResponse
        )
        return result

    def _job(self, **params):
        resource_path = "jobs" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.JobListResponse
        )
        return result

    def _role(self, **params):
        resource_path = "roles" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.RoleListResponse
        )
        return result

    def _usage_record(self, **params):
        resource_path = "usageRecords" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.UsageRecordListResponse
        )
        return result

    def _user(self, **params):
        resource_path = "users" + self._get_url_param_string(params, [])
        result = self._get_resource(
            resource_path, response_model=models.UserListResponse
        )
        return result



from typing import List, Optional, Tuple
from uuid import UUID

from pydantic import BaseModel

from lqs.interface.core.models.__common__ import (
    DataResponseModel,
    PaginationModel,
    ProcessModel,
    ProcessCreateRequest,
    ProcessUpdateRequest,
    optional_field,
    Int64,
)


class IngestionPartIndexEntry(BaseModel):
    topic_id: str
    data_offset: int
    data_length: int
    chunk_compression: Optional[str]
    chunk_offset: Optional[int]
    chunk_length: Optional[int]
    timestamp: Int64


# TODO: in Python 3.11, we can use Tuple[*get_type_hints(IngestionPartIndex).values()]
IngestionPartIndexTuple = Tuple[
    str, int, int, Optional[str], Optional[int], Optional[int], Int64
]


class IngestionPartIndex(BaseModel):
    ingestion_part_id: UUID
    index: Optional[List[IngestionPartIndexTuple]]


class IngestionPartIndexCreateRequest(BaseModel):
    ingestion_part_id: UUID
    index: Optional[List[IngestionPartIndexTuple]] = None


class IngestionPartIndexUpdateRequest(BaseModel):
    index: Optional[List[IngestionPartIndexTuple]] = optional_field


class IngestionPartIndexDataResponse(DataResponseModel[IngestionPartIndex]):
    pass


class IngestionPartIndexListResponse(PaginationModel[IngestionPartIndex]):
    pass


# Ingestion Part


class IngestionPart(ProcessModel["IngestionPart"]):
    sequence: int
    ingestion_id: UUID
    source: Optional[str]
    index: Optional[List[IngestionPartIndexTuple]]

    log_id: Optional[UUID]
    group_id: Optional[UUID]


class IngestionPartCreateRequest(ProcessCreateRequest):
    sequence: int
    source: Optional[str] = None
    index: Optional[List[IngestionPartIndexTuple]] = None


class IngestionPartUpdateRequest(ProcessUpdateRequest):
    sequence: int = optional_field
    source: Optional[str] = optional_field
    index: Optional[List[IngestionPartIndexTuple]] = optional_field


class IngestionPartListResponse(PaginationModel[IngestionPart]):
    pass


class IngestionPartDataResponse(DataResponseModel[IngestionPart]):
    pass



from typing import Optional
from datetime import datetime
from uuid import UUID
from abc import abstractmethod

import lqs.interface.dsm.models as models
from lqs.interface.base.list import ListInterface as BaseListInterface


class ListInterface(BaseListInterface):
    @abstractmethod
    def _announcement(self, **kwargs) -> models.AnnouncementListResponse:
        pass

    def announcement(
        self,
        id: Optional[UUID] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        datastore_id: Optional[UUID] = None,
        subject: Optional[str] = None,
        subject_like: Optional[str] = None,
        content: Optional[str] = None,
        content_like: Optional[str] = None,
        context: Optional[dict] = None,
        status: Optional[str] = None,
        status_like: Optional[str] = None,
        starts_at_gt: Optional[int] = None,
        starts_at_lt: Optional[datetime] = None,
        starts_at_gte: Optional[datetime] = None,
        starts_at_lte: Optional[datetime] = None,
        ends_at_gt: Optional[datetime] = None,
        ends_at_lt: Optional[datetime] = None,
        ends_at_gte: Optional[datetime] = None,
        ends_at_lte: Optional[datetime] = None,
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._announcement(
            id=id,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            datastore_id=datastore_id,
            subject=subject,
            subject_like=subject_like,
            content=content,
            content_like=content_like,
            context=context,
            status=status,
            status_like=status_like,
            starts_at_gt=starts_at_gt,
            starts_at_lt=starts_at_lt,
            starts_at_gte=starts_at_gte,
            starts_at_lte=starts_at_lte,
            ends_at_gt=ends_at_gt,
            ends_at_lt=ends_at_lt,
            ends_at_gte=ends_at_gte,
            ends_at_lte=ends_at_lte,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def announcements(self, **kwargs):
        return self.announcement(**kwargs)

    @abstractmethod
    def _comment(self, **kwargs) -> models.CommentListResponse:
        pass

    def comment(
        self,
        id: Optional[UUID] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        user_id: Optional[UUID] = None,
        datastore_id: Optional[UUID] = None,
        resource_type: Optional[str] = None,
        resource_id: Optional[UUID] = None,
        subject: Optional[str] = None,
        subject_like: Optional[str] = None,
        content: Optional[str] = None,
        content_like: Optional[str] = None,
        context: Optional[dict] = None,
        status: Optional[str] = None,
        status_like: Optional[str] = None,
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._comment(
            id=id,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            user_id=user_id,
            datastore_id=datastore_id,
            resource_type=resource_type,
            resource_id=resource_id,
            subject=subject,
            subject_like=subject_like,
            content=content,
            content_like=content_like,
            context=context,
            status=status,
            status_like=status_like,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def comments(self, **kwargs):
        return self.comment(**kwargs)

    @abstractmethod
    def _configuration(self, **kwargs) -> models.ConfigurationListResponse:
        pass

    def configuration(
        self,
        id: Optional[UUID] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        value: Optional[dict] = None,
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        note: Optional[str] = None,
        note_like: Optional[str] = None,
        default: Optional[bool] = None,
        disabled: Optional[bool] = None,
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._configuration(
            id=id,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            value=value,
            name=name,
            name_like=name_like,
            note=note,
            note_like=note_like,
            default=default,
            disabled=disabled,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def configurations(self, **kwargs):
        return self.configuration(**kwargs)

    @abstractmethod
    def _datastore(self, **kwargs) -> models.DataStoreListResponse:
        pass

    def datastore(
        self,
        id: Optional[UUID] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        name: Optional[str] = None,
        name_like: Optional[str] = None,
        note_like: Optional[str] = None,
        config_filter: Optional[str] = None,
        owner_id: Optional[UUID] = None,
        endpoint: Optional[str] = None,
        endpoint_like: Optional[str] = None,
        disabled: Optional[bool] = None,
        plan: Optional[str] = None,
        plan_like: Optional[str] = None,
        plan_null: Optional[bool] = None,
        record_count: Optional[int] = None,
        record_count_gt: Optional[int] = None,
        record_count_lt: Optional[int] = None,
        record_count_gte: Optional[int] = None,
        record_count_lte: Optional[int] = None,
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._datastore(
            id=id,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            name=name,
            name_like=name_like,
            note_like=note_like,
            config_filter=config_filter,
            owner_id=owner_id,
            endpoint=endpoint,
            endpoint_like=endpoint_like,
            disabled=disabled,
            plan=plan,
            plan_like=plan_like,
            plan_null=plan_null,
            record_count=record_count,
            record_count_gt=record_count_gt,
            record_count_lt=record_count_lt,
            record_count_gte=record_count_gte,
            record_count_lte=record_count_lte,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def datastores(self, **kwargs):
        return self.datastore(**kwargs)

    @abstractmethod
    def _datastore_association(
        self, **kwargs
    ) -> models.DataStoreAssociationListResponse:
        pass

    def datastore_association(
        self,
        id: Optional[UUID] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        user_id: Optional[UUID] = None,
        datastore_id: Optional[UUID] = None,
        datastore_user_id: Optional[UUID] = None,
        manager: Optional[bool] = None,
        disabled: Optional[bool] = None,
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._datastore_association(
            id=id,
            include_count=include_count,
            user_id=user_id,
            datastore_id=datastore_id,
            datastore_user_id=datastore_user_id,
            manager=manager,
            disabled=disabled,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def datastore_associations(self, **kwargs):
        return self.datastore_association(**kwargs)

    @abstractmethod
    def _event(self, **kwargs) -> models.EventListResponse:
        pass

    def event(
        self,
        id: Optional[UUID] = None,
        previous_state: Optional[str] = None,
        current_state: Optional[str] = None,
        process_type: Optional[str] = None,
        resource_id: Optional[UUID] = None,
        workflow_id: Optional[UUID] = None,
        hook_id: Optional[UUID] = None,
        datastore_id: Optional[UUID] = None,
        datastore_endpoint: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._event(
            id=id,
            previous_state=previous_state,
            current_state=current_state,
            process_type=process_type,
            resource_id=resource_id,
            workflow_id=workflow_id,
            hook_id=hook_id,
            datastore_id=datastore_id,
            datastore_endpoint=datastore_endpoint,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def events(self, **kwargs):
        return self.event(**kwargs)

    @abstractmethod
    def _job(self, **kwargs) -> models.JobListResponse:
        pass

    def job(
        self,
        id: Optional[UUID] = None,
        event_id: Optional[UUID] = None,
        process_type: Optional[str] = None,
        resource_id: Optional[UUID] = None,
        datastore_id: Optional[UUID] = None,
        datastore_endpoint: Optional[str] = None,
        state: Optional[str] = None,
        include_count: Optional[bool] = True,
        error_filter: Optional[str] = None,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._job(
            id=id,
            event_id=event_id,
            process_type=process_type,
            resource_id=resource_id,
            datastore_id=datastore_id,
            datastore_endpoint=datastore_endpoint,
            state=state,
            include_count=include_count,
            error_filter=error_filter,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def jobs(self, **kwargs):
        return self.job(**kwargs)

    @abstractmethod
    def _usage_record(self, **kwargs) -> models.UsageRecordListResponse:
        pass

    def usage_record(
        self,
        timestamp: Optional[models.Int64] = None,
        datastore_id: Optional[UUID] = None,
        user_id: Optional[UUID] = None,
        datastore_id_null: Optional[bool] = None,
        user_id_null: Optional[bool] = None,
        timestamp_gt: Optional[models.Int64] = None,
        timestamp_lt: Optional[models.Int64] = None,
        timestamp_gte: Optional[models.Int64] = None,
        timestamp_lte: Optional[models.Int64] = None,
        duration: Optional[models.Int64] = None,
        duration_gt: Optional[models.Int64] = None,
        duration_lt: Optional[models.Int64] = None,
        duration_gte: Optional[models.Int64] = None,
        duration_lte: Optional[models.Int64] = None,
        category: Optional[str] = None,
        usage_data_filter: Optional[str] = None,
        frequency: Optional[float] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "timestamp",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ):
        return self._usage_record(
            timestamp=timestamp,
            datastore_id=datastore_id,
            user_id=user_id,
            datastore_id_null=datastore_id_null,
            user_id_null=user_id_null,
            timestamp_gt=timestamp_gt,
            timestamp_lt=timestamp_lt,
            timestamp_gte=timestamp_gte,
            timestamp_lte=timestamp_lte,
            duration=duration,
            duration_gt=duration_gt,
            duration_lt=duration_lt,
            duration_gte=duration_gte,
            duration_lte=duration_lte,
            category=category,
            usage_data_filter=usage_data_filter,
            frequency=frequency,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def usage_records(self, **kwargs):
        return self.usage_record(**kwargs)

    @abstractmethod
    def _user(self, **kwargs) -> models.UserListResponse:
        pass

    def user(
        self,
        username: Optional[str] = None,
        username_like: Optional[str] = None,
        role_id: Optional[UUID] = None,
        managed: Optional[bool] = None,
        disabled: Optional[bool] = None,
        admin: Optional[bool] = None,
        external_id: Optional[str] = None,
        context_filter: Optional[str] = None,
        first_name: Optional[str] = None,
        first_name_like: Optional[str] = None,
        last_name: Optional[str] = None,
        last_name_like: Optional[str] = None,
        phone: Optional[str] = None,
        phone_like: Optional[str] = None,
        company: Optional[str] = None,
        company_like: Optional[str] = None,
        job_title: Optional[str] = None,
        job_title_like: Optional[str] = None,
        language: Optional[str] = None,
        language_like: Optional[str] = None,
        time_zone: Optional[str] = None,
        time_zone_like: Optional[str] = None,
        address_line_1: Optional[str] = None,
        address_line_1_like: Optional[str] = None,
        address_line_2: Optional[str] = None,
        address_line_2_like: Optional[str] = None,
        city: Optional[str] = None,
        city_like: Optional[str] = None,
        state: Optional[str] = None,
        state_like: Optional[str] = None,
        postal_code: Optional[str] = None,
        postal_code_like: Optional[str] = None,
        country: Optional[str] = None,
        country_like: Optional[str] = None,
        include_count: Optional[bool] = True,
        offset: Optional[int] = 0,
        limit: Optional[int] = 100,
        order: Optional[str] = "created_at",
        sort: Optional[str] = "ASC",
        created_by: Optional[UUID] = None,
        updated_by: Optional[UUID] = None,
        deleted_by: Optional[UUID] = None,
        updated_by_null: Optional[bool] = None,
        deleted_by_null: Optional[bool] = None,
        updated_at_null: Optional[bool] = None,
        deleted_at_null: Optional[bool] = None,
        created_at_lte: Optional[datetime] = None,
        updated_at_lte: Optional[datetime] = None,
        deleted_at_lte: Optional[datetime] = None,
        created_at_gte: Optional[datetime] = None,
        updated_at_gte: Optional[datetime] = None,
        deleted_at_gte: Optional[datetime] = None,
    ) -> models.UserListResponse:
        return self._user(
            username=username,
            username_like=username_like,
            role_id=role_id,
            managed=managed,
            disabled=disabled,
            admin=admin,
            external_id=external_id,
            context_filter=context_filter,
            first_name=first_name,
            first_name_like=first_name_like,
            last_name=last_name,
            last_name_like=last_name_like,
            phone=phone,
            phone_like=phone_like,
            company=company,
            company_like=company_like,
            job_title=job_title,
            job_title_like=job_title_like,
            language=language,
            language_like=language_like,
            time_zone=time_zone,
            time_zone_like=time_zone_like,
            address_line_1=address_line_1,
            address_line_1_like=address_line_1_like,
            address_line_2=address_line_2,
            address_line_2_like=address_line_2_like,
            city=city,
            city_like=city_like,
            state=state,
            state_like=state_like,
            postal_code=postal_code,
            postal_code_like=postal_code_like,
            country=country,
            country_like=country_like,
            include_count=include_count,
            offset=offset,
            limit=limit,
            order=order,
            sort=sort,
            created_by=created_by,
            updated_by=updated_by,
            deleted_by=deleted_by,
            updated_by_null=updated_by_null,
            deleted_by_null=deleted_by_null,
            updated_at_null=updated_at_null,
            deleted_at_null=deleted_at_null,
            created_at_lte=created_at_lte,
            updated_at_lte=updated_at_lte,
            deleted_at_lte=deleted_at_lte,
            created_at_gte=created_at_gte,
            updated_at_gte=updated_at_gte,
            deleted_at_gte=deleted_at_gte,
        )

    def users(self, **kwargs) -> models.UserListResponse:
        return self.user(**kwargs)



from abc import abstractmethod
from typing import List, Optional
from uuid import UUID

from lqs.interface.base.create import CreateInterface as BaseCreateInterface
import lqs.interface.core.models as models
from lqs.interface.core.models import ProcessState


class CreateInterface(BaseCreateInterface):
    @abstractmethod
    def _digestion(self, **kwargs) -> models.DigestionDataResponse:
        pass

    def digestion(
        self,
        log_id: UUID,
        name: Optional[str] = None,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        locked: Optional[bool] = False,
        workflow_id: Optional[UUID] = None,
        workflow_context: Optional[dict] = None,
        state: ProcessState = ProcessState.ready,
        lock_token: Optional[str] = None,
    ):
        """
        Create a digestion.

        :param log_id: The ID of the log to which the digestion should be added.
        :type log_id: str
        :param name: The name of the digestion. Defaults to None.
        :type name: str, optional
        :param context: The context to use for the digestion. Defaults to None.
        :type context: dict, optional
        :param note: A note about the digestion. Defaults to None.
        :type note: str, optional
        :param locked: Whether the digestion is locked. Defaults to False.
        :type locked: bool, optional
        :param workflow_id: The ID of the workflow to use for the digestion. Defaults to None.
        :type workflow_id: str, optional
        :param workflow_context: The context to use for the workflow. Defaults to None.
        :type workflow_context: dict, optional
        :param state: The state of the digestion. Defaults to ProcessState.ready.
        :type state: ProcessState, optional
        :returns: A data response with the created digestion.
        :rtype: DigestionDataResponse
        """

        return self._digestion(
            log_id=log_id,
            name=name,
            note=note,
            context=context,
            locked=locked,
            workflow_id=workflow_id,
            workflow_context=workflow_context,
            state=state,
            lock_token=lock_token,
        )

    def _digestion_by_model(
        self,
        data: models.DigestionCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.digestion(**data.model_dump(), lock_token=lock_token)

    @abstractmethod
    def _digestion_part(self, **kwargs) -> models.DigestionPartDataResponse:
        pass

    def digestion_part(
        self,
        digestion_id: UUID,
        sequence: int,
        locked: Optional[bool] = False,
        workflow_id: Optional[UUID] = None,
        workflow_context: Optional[dict] = None,
        state: ProcessState = ProcessState.ready,
        index: Optional[List[models.DigestionPartIndex]] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create a digestion part.

        :param digestion_id: The ID of the digestion to which the digestion part should be added.
        :type digestion_id: str
        :param sequence: The sequence of the digestion part.
        :type sequence: int
        :param locked: Whether the digestion part is locked. Defaults to False.
        :type locked: bool, optional
        :param workflow_id: The ID of the workflow to use for the digestion part. Defaults to None.
        :type workflow_id: str, optional
        :param workflow_context: The context to use for the workflow. Defaults to None.
        :type workflow_context: dict, optional
        :param state: The state of the digestion part. Defaults to ProcessState.ready.
        :type state: ProcessState, optional
        :param index: The index of the digestion part. Defaults to None.
        :type index: int, optional
        :returns: A data response with the created digestion part.
        :rtype: DigestionPartDataResponse
        """
        return self._digestion_part(
            digestion_id=digestion_id,
            sequence=sequence,
            locked=locked,
            workflow_id=workflow_id,
            workflow_context=workflow_context,
            state=state,
            index=index,
            lock_token=lock_token,
        )

    def _digestion_part_by_model(
        self,
        digestion_id: UUID,
        data: models.DigestionPartCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.digestion_part(
            digestion_id=digestion_id, **data.model_dump(), lock_token=lock_token
        )

    @abstractmethod
    def _digestion_topic(self, **kwargs) -> models.DigestionTopicDataResponse:
        pass

    def digestion_topic(
        self,
        digestion_id: UUID,
        topic_id: UUID,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
        frequency: Optional[float] = None,
        query_data_filter: Optional[dict] = None,
        context_filter: Optional[dict] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create a digestion topic.

        :param digestion_id: The ID of the digestion to which the digestion topic should be added.
        :type digestion_id: str
        :param topic_id: The ID of the topic to be digested.
        :type topic_id: str
        :param start_time: The start time of the digestion topic. Defaults to None.
        :type start_time: int, optional
        :param end_time: The end time of the digestion topic. Defaults to None.
        :type end_time: int, optional
        :param frequency: The frequency of the digestion topic. Defaults to None.
        :type frequency: float, optional
        :param query_data_filter: The data filter of the digestion topic. Defaults to None.
        :type query_data_filter: dict, optional
        :param context_filter: The context filter of the digestion topic. Defaults to None.
        :type context_filter: dict, optional
        :returns: A data response with the created digestion topic.
        :rtype: DigestionTopicDataResponse
        """
        return self._digestion_topic(
            digestion_id=digestion_id,
            topic_id=topic_id,
            start_time=start_time,
            end_time=end_time,
            frequency=frequency,
            query_data_filter=query_data_filter,
            context_filter=context_filter,
            lock_token=lock_token,
        )

    def _digestion_topic_by_model(
        self,
        digestion_id: UUID,
        data: models.DigestionTopicCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.digestion_topic(
            digestion_id=digestion_id, **data.model_dump(), lock_token=lock_token
        )

    @abstractmethod
    def _group(self, **kwargs) -> models.GroupDataResponse:
        pass

    def group(
        self,
        name: str,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        locked: Optional[bool] = False,
        default_workflow_id: Optional[UUID] = None,
    ):
        """
        Create a group.

        :param name: The name of the group.
        :type name: str
        :param note: A note about the group. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the group. Defaults to None.
        :type context: dict, optional
        :param locked: Whether the group is locked. Defaults to False.
        :type locked: bool, optional
        :param default_workflow_id: The ID of the default workflow for the group. Defaults to None.
        :type default_workflow_id: str, optional
        :returns: A data response with the created group.
        :rtype: GroupDataResponse
        """
        return self._group(
            name=name,
            note=note,
            context=context,
            locked=locked,
            default_workflow_id=default_workflow_id,
        )

    def _group_by_model(self, data: models.GroupCreateRequest):
        return self.group(**data.model_dump())

    @abstractmethod
    def _hook(self, **kwargs) -> models.HookDataResponse:
        pass

    def hook(
        self,
        workflow_id: UUID,
        trigger_process: str,
        trigger_state: str,
        name: Optional[str] = None,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        managed: Optional[bool] = False,
        disabled: Optional[bool] = False,
        uri: Optional[str] = None,
        secret: Optional[str] = None,
    ):
        """
        Create a hook.

        :param workflow_id: The ID of the workflow to which the hook should be added.
        :type workflow_id: str
        :param trigger_process: The process to trigger.
        :type trigger_process: str
        :param trigger_state: The state to trigger.
        :type trigger_state: str
        :param name: The name of the hook. Defaults to None.
        :type name: str, optional
        :param note: A note about the hook. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the hook. Defaults to None.
        :type context: dict, optional
        :param managed: Whether the hook is managed. Defaults to False.
        :type managed: bool, optional
        :param disabled: Whether the hook is disabled. Defaults to False.
        :type disabled: bool, optional
        :param uri: The URI of the hook. Defaults to None.
        :type uri: str, optional
        :param secret: The secret of the hook. Defaults to None.
        :type secret: str, optional
        :returns: A data response with the created hook.
        :rtype: HookDataResponse
        """
        return self._hook(
            workflow_id=workflow_id,
            trigger_process=trigger_process,
            trigger_state=trigger_state,
            name=name,
            note=note,
            context=context,
            managed=managed,
            disabled=disabled,
            uri=uri,
            secret=secret,
        )

    def _hook_by_model(self, workflow_id: UUID, data: models.HookCreateRequest):
        return self.hook(workflow_id=workflow_id, **data.model_dump())

    @abstractmethod
    def _ingestion(self, **kwargs) -> models.IngestionDataResponse:
        pass

    def ingestion(
        self,
        log_id: UUID,
        name: Optional[str] = None,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        object_store_id: Optional[UUID] = None,
        object_key: Optional[str] = None,
        locked: Optional[bool] = False,
        workflow_id: Optional[UUID] = None,
        workflow_context: Optional[dict] = None,
        state: ProcessState = ProcessState.ready,
        lock_token: Optional[str] = None,
    ):
        """
        Create an ingestion.

        :param log_id: The ID of the log to which the ingestion should be added.
        :type log_id: str
        :param name: The name of the ingestion. Defaults to None.
        :type name: str, optional
        :param note: A note about the ingestion. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the ingestion. Defaults to None.
        :type context: dict, optional
        :param object_store_id: The ID of the object store to use for the ingestion. Defaults to None.
        :type object_store_id: str, optional
        :param object_key: The key of the object to use for the ingestion. Defaults to None.
        :type object_key: str, optional
        :param locked: Whether the ingestion is locked. Defaults to False.
        :type locked: bool, optional
        :param workflow_id: The ID of the workflow to use for the ingestion. Defaults to None.
        :type workflow_id: str, optional
        :param workflow_context: The context to use for the workflow. Defaults to None.
        :type workflow_context: dict, optional
        :param state: The state of the ingestion. Defaults to ProcessState.ready.
        :type state: ProcessState, optional
        :returns: A data response with the created ingestion.
        :rtype: IngestionDataResponse
        """
        return self._ingestion(
            log_id=log_id,
            name=name,
            note=note,
            context=context,
            object_store_id=object_store_id,
            object_key=object_key,
            locked=locked,
            workflow_id=workflow_id,
            workflow_context=workflow_context,
            state=state,
            lock_token=lock_token,
        )

    def _ingestion_by_model(
        self,
        data: models.IngestionCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.ingestion(**data.model_dump(), lock_token=lock_token)

    @abstractmethod
    def _ingestion_part(self, **kwargs) -> models.IngestionPartDataResponse:
        pass

    def ingestion_part(
        self,
        ingestion_id: UUID,
        sequence: int,
        source: Optional[str] = None,
        locked: Optional[bool] = False,
        workflow_id: Optional[UUID] = None,
        workflow_context: Optional[dict] = None,
        state: ProcessState = ProcessState.ready,
        index: Optional[List[models.IngestionPartIndex]] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create an ingestion part.

        :param ingestion_id: The ID of the ingestion to which the ingestion part should be added.
        :type ingestion_id: str
        :param sequence: The sequence of the ingestion part.
        :type sequence: int
        :param source: The source of the ingestion part. Defaults to None.
        :type source: str, optional
        :param locked: Whether the ingestion part is locked. Defaults to False.
        :type locked: bool, optional
        :param workflow_id: The ID of the workflow to use for the ingestion part. Defaults to None.
        :type workflow_id: str, optional
        :param workflow_context: The context to use for the workflow. Defaults to None.
        :type workflow_context: dict, optional
        :param state: The state of the ingestion part. Defaults to ProcessState.queued.
        :type state: ProcessState, optional
        :param index: The index of the ingestion part. Defaults to None.
        :type index: int, optional
        :returns: A data response with the created ingestion part.
        :rtype: IngestionPartDataResponse
        """
        return self._ingestion_part(
            ingestion_id=ingestion_id,
            sequence=sequence,
            source=source,
            locked=locked,
            workflow_id=workflow_id,
            workflow_context=workflow_context,
            state=state,
            index=index,
            lock_token=lock_token,
        )

    def _ingestion_part_by_model(
        self,
        ingestion_id: UUID,
        data: models.IngestionPartCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.ingestion_part(
            ingestion_id=ingestion_id, **data.model_dump(), lock_token=lock_token
        )

    @abstractmethod
    def _label(self, **kwargs) -> models.LabelDataResponse:
        pass

    def label(self, value: str, note: Optional[str] = None):
        """
        Create a label.

        :param value: The value of the label.
        :type value: str
        :param note: A note about the label. Defaults to None.
        :type note: str, optional
        :returns: A data response with the created label.
        :rtype: LabelDataResponse
        """
        return self._label(
            value=value,
            note=note,
        )

    def _label_by_model(self, data: models.LabelCreateRequest):
        return self.label(**data.model_dump())

    @abstractmethod
    def _log(self, **kwargs) -> models.LogDataResponse:
        pass

    def log(
        self,
        group_id: UUID,
        name: str,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        locked: Optional[bool] = False,
        default_workflow_id: Optional[UUID] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create a log.

        :param group_id: The ID of the group to which the log should be added.
        :type group_id: str
        :param name: The name of the log.
        :type name: str
        :param note: A note about the log. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the log. Defaults to None.
        :type context: dict, optional
        :param locked: Whether the log is locked. Defaults to False.
        :type locked: bool, optional
        :param default_workflow_id: The ID of the default workflow for the log. Defaults to None.
        :type default_workflow_id: str, optional
        :returns: A data response with the created log.
        :rtype: LogDataResponse
        """
        return self._log(
            group_id=group_id,
            name=name,
            note=note,
            context=context,
            locked=locked,
            default_workflow_id=default_workflow_id,
            lock_token=lock_token,
        )

    def _log_by_model(
        self,
        data: models.LogCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.log(**data.model_dump(), lock_token=lock_token)

    @abstractmethod
    def _log_object(self, **kwargs) -> models.ObjectDataResponse:
        pass

    def log_object(
        self,
        key: str,
        log_id: UUID,
        content_type: Optional[str] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create a log object.

        :param key: The key of the log object.
        :type key: str
        :param log_id: The ID of the log to which the log object should be added.
        :type log_id: str
        :param content_type: The content type of the log object. Defaults to None.
        :type content_type: str, optional
        :returns: A data response with the created log object.
        :rtype: LogObjectDataResponse
        """
        return self._log_object(
            key=key,
            log_id=log_id,
            content_type=content_type,
            lock_token=lock_token,
        )

    def _log_object_by_model(
        self,
        log_id: UUID,
        data: models.ObjectCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.log_object(
            log_id=log_id, **data.model_dump(), lock_token=lock_token
        )

    @abstractmethod
    def _log_object_part(self, **kwargs) -> models.ObjectPartDataResponse:
        pass

    def log_object_part(
        self,
        object_key: str,
        size: int,
        log_id: UUID,
        part_number: Optional[int] = None,
    ):
        """
        Create a log object part.

        :param object_key: The key of the log object to which the log object part should be added.
        :type object_key: str
        :param size: The size of the log object part.
        :type size: int
        :param log_id: The ID of the log to which the log object part should be added.
        :type log_id: str
        :param part_number: The part number of the log object part. Defaults to None.
        :type part_number: int, optional
        :returns: A data response with the created log object part.
        :rtype: LogObjectPartDataResponse
        """
        return self._log_object_part(
            object_key=object_key,
            log_id=log_id,
            part_number=part_number,
            size=size,
        )

    def _log_object_part_by_model(
        self, object_key: str, log_id: UUID, data: models.ObjectPartCreateRequest
    ):
        return self.log_object_part(
            object_key=object_key, log_id=log_id, **data.model_dump()
        )

    @abstractmethod
    def _object(self, **kwargs) -> models.ObjectDataResponse:
        pass

    def object(
        self,
        key: str,
        object_store_id: UUID,
        content_type: Optional[str] = None,
    ):
        """
        Create an object.

        :param key: The key of the object.
        :type key: str
        :param object_store_id: The ID of the object store to which the object should be added.
        :type object_store_id: str
        :param content_type: The content type of the object. Defaults to None.
        :type content_type: str, optional
        :returns: A data response with the created object.
        :rtype: ObjectDataResponse
        """
        return self._object(
            key=key,
            object_store_id=object_store_id,
            content_type=content_type,
        )

    def _object_by_model(self, object_store_id: UUID, data: models.ObjectCreateRequest):
        return self.object(object_store_id=object_store_id, **data.model_dump())

    @abstractmethod
    def _object_part(self, **kwargs) -> models.ObjectPartDataResponse:
        pass

    def object_part(
        self,
        object_key: str,
        size: int,
        object_store_id: UUID,
        part_number: Optional[int] = None,
    ):
        """
        Create an object part.

        :param object_key: The key of the object to which the object part should be added.
        :type object_key: str
        :param size: The size of the object part.
        :type size: int
        :param object_store_id: The ID of the object store to which the object part should be added.
        :type object_store_id: str
        :param part_number: The part number of the object part. Defaults to None.
        :type part_number: int, optional
        :returns: A data response with the created object part.
        :rtype: ObjectPartDataResponse
        """
        return self._object_part(
            object_key=object_key,
            object_store_id=object_store_id,
            part_number=part_number,
            size=size,
        )

    def _object_part_by_model(
        self,
        object_key: str,
        object_store_id: UUID,
        data: models.ObjectPartCreateRequest,
    ):
        return self.object_part(
            object_key=object_key, object_store_id=object_store_id, **data.model_dump()
        )

    @abstractmethod
    def _object_store(self, **kwargs) -> models.ObjectStoreDataResponse:
        pass

    def object_store(
        self,
        bucket_name: str,
        access_key_id: Optional[str] = None,
        secret_access_key: Optional[str] = None,
        region_name: Optional[str] = None,
        endpoint_url: Optional[str] = None,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        disabled: Optional[bool] = False,
    ):
        """
        Create an object store.

        :param bucket_name: The name of the bucket.
        :type bucket_name: str
        :param access_key_id: The access key ID of the object store. Defaults to None.
        :type access_key_id: str, optional
        :param secret_access_key: The secret access key of the object store. Defaults to None.
        :type secret_access_key: str, optional
        :param region_name: The region name of the object store. Defaults to None.
        :type region_name: str, optional
        :param endpoint_url: The endpoint URL of the object store. Defaults to None.
        :type endpoint_url: str, optional
        :param note: A note about the object store. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the object store. Defaults to None.
        :type context: dict, optional
        :param disabled: Whether the object store is disabled. Defaults to False.
        :type disabled: bool, optional
        :returns: A data response with the created object store.
        :rtype: ObjectStoreDataResponse
        """
        return self._object_store(
            bucket_name=bucket_name,
            access_key_id=access_key_id,
            secret_access_key=secret_access_key,
            region_name=region_name,
            endpoint_url=endpoint_url,
            note=note,
            context=context,
            disabled=disabled,
        )

    def _object_store_by_model(self, data: models.ObjectStoreCreateRequest):
        return self.object_store(**data.model_dump())

    @abstractmethod
    def _query(self, **kwargs) -> models.QueryDataResponse:
        pass

    def query(
        self,
        log_id: UUID,
        name: Optional[str] = None,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        statement: Optional[str] = None,
        parameters: Optional[dict] = None,
    ):
        """
        Create a query.

        :param log_id: The ID of the log to which the query should be added.
        :type log_id: str
        :param name: The name of the query. Defaults to None.
        :type name: str, optional
        :param note: A note about the query. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the query. Defaults to None.
        :type context: dict, optional
        :param statement: The statement of the query. Defaults to None.
        :type statement: str, optional
        :param parameters: The parameters of the query. Defaults to None.
        :type parameters: dict, optional
        :returns: A data response with the created query.
        :rtype: QueryDataResponse
        """
        return self._query(
            log_id=log_id,
            name=name,
            note=note,
            context=context,
            statement=statement,
            parameters=parameters,
        )

    def _query_by_model(self, log_id: UUID, data: models.QueryCreateRequest):
        return self.query(log_id=log_id, **data.model_dump())

    @abstractmethod
    def _record(self, **kwargs) -> models.RecordDataResponse:
        pass

    def record(
        self,
        timestamp: int,
        topic_id: UUID,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        locked: Optional[bool] = False,
        query_data: Optional[dict] = None,
        auxiliary_data: Optional[dict] = None,
        data_offset: Optional[int] = None,
        data_length: Optional[int] = None,
        chunk_compression: Optional[str] = None,
        chunk_offset: Optional[int] = None,
        chunk_length: Optional[int] = None,
        source: Optional[str] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create a record.

        :param timestamp: The timestamp of the record.
        :type timestamp: int
        :param topic_id: The ID of the topic to which the record should be added.
        :type topic_id: str
        :param note: A note about the record. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the record. Defaults to None.
        :type context: dict, optional
        :param locked: Whether the record is locked. Defaults to False.
        :type locked: bool, optional
        :param query_data: A JSON representation of the record's message data which is queryable. Defaults to None.
        :type query_data: dict, optional
        :param auxiliary_data: A JSON representation of the record's message data which is not queryable. Defaults to None.
        :type auxiliary_data: dict, optional
        :param data_offset: The data offset of the record. Defaults to None.
        :type data_offset: int, optional
        :param data_length: The data length of the record. Defaults to None.
        :type data_length: int, optional
        :param chunk_compression: The chunk compression of the record. Defaults to None.
        :type chunk_compression: str, optional
        :param chunk_offset: The chunk offset of the record. Defaults to None.
        :type chunk_offset: int, optional
        :param chunk_length: The chunk length of the record. Defaults to None.
        :type chunk_length: int, optional
        :param source: The source of the record. Defaults to None.
        :type source: str, optional
        :returns: A data response with the created record.
        :rtype: RecordDataResponse
        """
        return self._record(
            timestamp=timestamp,
            topic_id=topic_id,
            note=note,
            context=context,
            locked=locked,
            query_data=query_data,
            auxiliary_data=auxiliary_data,
            data_offset=data_offset,
            data_length=data_length,
            chunk_compression=chunk_compression,
            chunk_offset=chunk_offset,
            chunk_length=chunk_length,
            source=source,
            lock_token=lock_token,
        )

    def _record_by_model(
        self,
        topic_id: UUID,
        data: models.RecordCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.record(
            topic_id=topic_id, **data.model_dump(), lock_token=lock_token
        )

    @abstractmethod
    def _tag(self, **kwargs) -> models.TagDataResponse:
        pass

    def tag(
        self,
        label_id: UUID,
        log_id: UUID,
        topic_id: Optional[UUID] = None,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        start_time: Optional[int] = None,
        end_time: Optional[int] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create a tag.

        :param label_id: The ID of the label to which the tag should be added.
        :type label_id: str
        :param log_id: The ID of the log to which the tag should be added.
        :type log_id: str
        :param topic_id: The ID of the topic to which the tag should be added. Defaults to None.
        :type topic_id: str, optional
        :param note: A note about the tag. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the tag. Defaults to None.
        :type context: dict, optional
        :param start_time: The start time of the tag. Defaults to None.
        :type start_time: int, optional
        :param end_time: The end time of the tag. Defaults to None.
        :type end_time: int, optional
        :returns: A data response with the created tag.
        :rtype: TagDataResponse
        """
        return self._tag(
            label_id=label_id,
            log_id=log_id,
            topic_id=topic_id,
            note=note,
            context=context,
            start_time=start_time,
            end_time=end_time,
            lock_token=lock_token,
        )

    def _tag_by_model(
        self,
        log_id: UUID,
        data: models.TagCreateRequest,
        lock_token: Optional[str] = None,
    ):
        return self.tag(log_id=log_id, **data.model_dump(), lock_token=lock_token)

    @abstractmethod
    def _topic(self, **kwargs) -> models.TopicDataResponse:
        pass

    def topic(
        self,
        log_id: UUID,
        name: str,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        associated_topic_id: Optional[UUID] = None,
        locked: Optional[bool] = False,
        strict: Optional[bool] = False,
        type_name: Optional[str] = None,
        type_encoding: Optional[str] = None,
        type_data: Optional[str] = None,
        type_schema: Optional[dict] = None,
        lock_token: Optional[str] = None,
    ):
        """
        Create a topic.

        :param log_id: The ID of the log to which the topic should be added.
        :type log_id: str
        :param name: The name of the topic.
        :type name: str
        :param note: A note about the topic. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the topic. Defaults to None.
        :type context: dict, optional
        :param associated_topic_id: The ID of the associated topic. Defaults to None.
        :type associated_topic_id: str, optional
        :param locked: Whether the topic is locked. Defaults to False.
        :type locked: bool, optional
        :param strict: Whether the topic is strict. Defaults to False.
        :type strict: bool, optional
        :param type_name: The type name of the topic. Defaults to None.
        :type type_name: str, optional
        :param type_encoding: The type encoding of the topic. Defaults to None.
        :type type_encoding: str, optional
        :param type_data: The type data of the topic. Defaults to None.
        :type type_data: str, optional
        :param type_schema: The type schema of the topic. Defaults to None.
        :type type_schema: str, optional
        :returns: A data response with the created topic.
        :rtype: TopicDataResponse
        """
        return self._topic(
            log_id=log_id,
            name=name,
            note=note,
            context=context,
            associated_topic_id=associated_topic_id,
            locked=locked,
            strict=strict,
            type_name=type_name,
            type_encoding=type_encoding,
            type_data=type_data,
            type_schema=type_schema,
            lock_token=lock_token,
        )

    def _topic_by_model(
        self, data: models.TopicCreateRequest, lock_token: Optional[str] = None
    ):
        return self.topic(**data.model_dump(), lock_token=lock_token)

    @abstractmethod
    def _workflow(self, **kwargs) -> models.WorkflowDataResponse:
        pass

    def workflow(
        self,
        name: str,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        default: Optional[bool] = False,
        disabled: Optional[bool] = False,
        managed: Optional[bool] = False,
        context_schema: Optional[dict] = None,
    ):
        """
        Create a workflow.

        :param name: The name of the workflow.
        :type name: str
        :param note: A note about the workflow. Defaults to None.
        :type note: str, optional
        :param context: The context to use for the workflow. Defaults to None.
        :type context: dict, optional
        :param default: Whether the workflow is default. Defaults to False.
        :type default: bool, optional
        :param disabled: Whether the workflow is disabled. Defaults to False.
        :type disabled: bool, optional
        :param managed: Whether the workflow is managed. Defaults to False.
        :type managed: bool, optional
        :param context_schema: The context schema of the workflow. Defaults to None.
        :type context_schema: dict, optional
        :returns: A data response with the created workflow.
        :rtype: WorkflowDataResponse
        """
        return self._workflow(
            name=name,
            note=note,
            context=context,
            default=default,
            disabled=disabled,
            managed=managed,
            context_schema=context_schema,
        )

    def _workflow_by_model(self, data: models.WorkflowCreateRequest):
        return self.workflow(**data.model_dump())



from typing import Optional
from uuid import UUID
from datetime import datetime

from pydantic import BaseModel

from lqs.interface.base.models.__common__ import (
    CommonModel,
    DataResponseModel,
    PaginationModel,
    optional_field,
)


class User(CommonModel):
    _repr_fields = ("id", "username")

    username: str
    role_id: Optional[UUID]
    admin: bool
    disabled: bool
    managed: bool
    external_id: Optional[str]
    context: Optional[dict]

    # Profile Fields
    first_name: Optional[str]
    last_name: Optional[str]
    phone: Optional[str]
    company: Optional[str]
    job_title: Optional[str]
    language: Optional[str]
    time_zone: Optional[str]
    address_line_1: Optional[str]
    address_line_2: Optional[str]
    city: Optional[str]
    state: Optional[str]
    postal_code: Optional[str]
    country: Optional[str]

    profile_picture: Optional[str]
    freshdesk_contact_id: Optional[int]

    _profile_fields = (
        "first_name",
        "last_name",
        "phone",
        "company",
        "job_title",
        "language",
        "time_zone",
        "address_line_1",
        "address_line_2",
        "city",
        "state",
        "postal_code",
        "country",
    )


class UserInternal(User):
    cognito_id: Optional[str]
    google_id: Optional[str]
    microsoft_id: Optional[str]
    apple_id: Optional[str]
    amazon_id: Optional[str]
    facebook_id: Optional[str]
    linkedin_id: Optional[str]
    github_id: Optional[str]
    stripe_customer_id: Optional[str]

    cookies_accepted_at: Optional[datetime]
    terms_accepted_at: Optional[datetime]
    privacy_accepted_at: Optional[datetime]
    marketing_accepted_at: Optional[datetime]


class UserDataResponse(DataResponseModel[User]):
    pass


class MeDataResponse(DataResponseModel[User]):
    data: Optional[User]


class UserListResponse(PaginationModel[User]):
    pass


class UserCreateRequest(BaseModel):
    username: str
    role_id: Optional[UUID] = None
    admin: bool = False
    disabled: bool = False
    managed: bool = False
    external_id: Optional[str] = None
    password: Optional[str] = None  # note: this is virtual
    context: Optional[dict] = None

    first_name: Optional[str] = None
    last_name: Optional[str] = None
    phone: Optional[str] = None
    company: Optional[str] = None
    job_title: Optional[str] = None
    language: Optional[str] = None
    time_zone: Optional[str] = None
    address_line_1: Optional[str] = None
    address_line_2: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    postal_code: Optional[str] = None
    country: Optional[str] = None


class UserUpdateRequest(BaseModel):
    username: str = optional_field
    role_id: Optional[UUID] = optional_field
    admin: bool = optional_field
    disabled: bool = optional_field
    external_id: Optional[str] = optional_field
    password: Optional[str] = optional_field  # note: this is virtual
    context: Optional[dict] = optional_field

    first_name: Optional[str] = optional_field
    last_name: Optional[str] = optional_field
    phone: Optional[str] = optional_field
    company: Optional[str] = optional_field
    job_title: Optional[str] = optional_field
    language: Optional[str] = optional_field
    time_zone: Optional[str] = optional_field
    address_line_1: Optional[str] = optional_field
    address_line_2: Optional[str] = optional_field
    city: Optional[str] = optional_field
    state: Optional[str] = optional_field
    postal_code: Optional[str] = optional_field
    country: Optional[str] = optional_field



from abc import abstractmethod
from typing import Optional
from uuid import UUID
from datetime import datetime
from lqs.interface.base.create import CreateInterface as BaseCreateInterface
import lqs.interface.dsm.models as models


class CreateInterface(BaseCreateInterface):
    @abstractmethod
    def _announcement(self, **kwargs) -> models.AnnouncementDataResponse:
        pass

    def announcement(
        self,
        datastore_id: Optional[UUID] = None,
        subject: Optional[str] = None,
        content: Optional[str] = None,
        context: Optional[dict] = None,
        status: Optional[str] = None,
        starts_at: Optional[datetime] = None,
        ends_at: Optional[datetime] = None,
    ):
        return self._announcement(
            datastore_id=datastore_id,
            subject=subject,
            content=content,
            context=context,
            status=status,
            starts_at=starts_at,
            ends_at=ends_at,
        )

    def _announcement_by_model(self, data: models.AnnouncementCreateRequest):
        return self.announcement(**data.model_dump())

    @abstractmethod
    def _comment(self, **kwargs) -> models.CommentDataResponse:
        pass

    def comment(
        self,
        user_id: Optional[UUID] = None,
        datastore_id: Optional[UUID] = None,
        resource_type: Optional[str] = None,
        resource_id: Optional[UUID] = None,
        subject: Optional[str] = None,
        content: Optional[str] = None,
        context: Optional[dict] = None,
    ):
        return self._comment(
            user_id=user_id,
            datastore_id=datastore_id,
            resource_type=resource_type,
            resource_id=resource_id,
            subject=subject,
            content=content,
            context=context,
        )

    def _comment_by_model(self, data: models.CommentCreateRequest):
        return self.comment(**data.model_dump())

    @abstractmethod
    def _configuration(self, **kwargs) -> models.ConfigurationDataResponse:
        pass

    def configuration(
        self,
        value: dict,
        name: Optional[str] = None,
        note: Optional[str] = None,
        default: bool = False,
        disabled: bool = False,
    ):
        return self._configuration(
            value=value,
            name=name,
            note=note,
            default=default,
            disabled=disabled,
        )

    def _configuration_by_model(self, data: models.ConfigurationCreateRequest):
        return self.configuration(**data.model_dump())

    @abstractmethod
    def _datastore(self, **kwargs) -> models.DataStoreDataResponse:
        pass

    def datastore(
        self,
        name: str,
        note: Optional[str] = None,
        context: Optional[dict] = None,
        owner_id: Optional[UUID] = None,
        config: Optional[dict] = None,
        version: Optional[str] = None,
        region: Optional[str] = None,
        endpoint: Optional[str] = None,
        disabled: bool = False,
    ):
        return self._datastore(
            name=name,
            note=note,
            context=context,
            owner_id=owner_id,
            config=config,
            version=version,
            region=region,
            endpoint=endpoint,
            disabled=disabled,
        )

    def _datastore_by_model(self, data: models.DataStoreCreateRequest):
        return self.datastore(**data.model_dump())

    @abstractmethod
    def _datastore_association(
        self, **kwargs
    ) -> models.DataStoreAssociationDataResponse:
        pass

    def datastore_association(
        self,
        user_id: UUID,
        datastore_id: UUID,
        manager: bool = False,
        disabled: bool = False,
        datastore_user_id: Optional[UUID] = None,
        datastore_username: Optional[str] = None,
        datastore_role_id: Optional[UUID] = None,
        datastore_admin: bool = False,
        datastore_disabled: bool = False,
    ):
        return self._datastore_association(
            user_id=user_id,
            datastore_id=datastore_id,
            manager=manager,
            disabled=disabled,
            datastore_user_id=datastore_user_id,
            datastore_username=datastore_username,
            datastore_role_id=datastore_role_id,
            datastore_admin=datastore_admin,
            datastore_disabled=datastore_disabled,
        )

    def _datastore_association_by_model(
        self, data: models.DataStoreAssociationCreateRequest
    ):
        return self.datastore_association(**data.model_dump())

    @abstractmethod
    def _event(self, **kwargs) -> models.EventDataResponse:
        pass

    def event(
        self,
        current_state: str,
        process_type: str,
        resource_id: UUID,
        previous_state: Optional[str] = None,
        workflow_id: Optional[UUID] = None,
        hook_id: Optional[UUID] = None,
        datastore_id: Optional[UUID] = None,
        datastore_endpoint: Optional[str] = None,
    ):
        return self._event(
            current_state=current_state,
            process_type=process_type,
            resource_id=resource_id,
            previous_state=previous_state,
            workflow_id=workflow_id,
            hook_id=hook_id,
            datastore_id=datastore_id,
            datastore_endpoint=datastore_endpoint,
        )

    def _event_by_model(self, data: models.EventCreateRequest):
        return self.event(**data.model_dump())

    @abstractmethod
    def _inference(self, **kwargs) -> models.InferenceDataResponse:
        pass

    def inference(
        self,
        datastore_id: UUID,
        topic_id: UUID,
        timestamp: int,
        pipeline_task: str,
        pipeline_model: Optional[str] = None,
        pipeline_revision: Optional[str] = None,
        max_image_size: Optional[int] = None,
        omit_output_fields: Optional[list[str]] = None,
    ):
        return self._inference(
            datastore_id=datastore_id,
            topic_id=topic_id,
            timestamp=timestamp,
            pipeline_task=pipeline_task,
            pipeline_model=pipeline_model,
            pipeline_revision=pipeline_revision,
            max_image_size=max_image_size,
            omit_output_fields=omit_output_fields,
        )

    def _inference_by_model(self, data: models.InferenceCreateRequest):
        return self.inference(**data.model_dump())

    @abstractmethod
    def _job(self, **kwargs) -> models.JobDataResponse:
        pass

    def job(
        self,
        type: str,
        process_type: str,
        resource_id: UUID,
        event_id: Optional[UUID] = None,
        datastore_id: Optional[UUID] = None,
        datastore_endpoint: Optional[str] = None,
        state: models.ProcessState = models.ProcessState.ready,
    ):
        return self._job(
            type=type,
            process_type=process_type,
            resource_id=resource_id,
            event_id=event_id,
            datastore_id=datastore_id,
            datastore_endpoint=datastore_endpoint,
            state=state,
        )

    def _job_by_model(self, data: models.JobCreateRequest):
        return self.job(**data.model_dump())

    @abstractmethod
    def _ticket(self, **kwargs) -> models.TicketDataResponse:
        pass

    def ticket(
        self,
        type: models.TicketType,
        description: str,
        email: str,
    ):
        return self._ticket(
            type=type,
            description=description,
            email=email,
        )

    def _ticket_by_model(self, data: models.TicketCreateRequest):
        return self.ticket(**data.model_dump())

    @abstractmethod
    def _usage_record(self, **kwargs) -> models.UsageRecordDataResponse:
        pass

    def usage_record(
        self,
        timestamp: models.Int64,
        datastore_id: Optional[UUID] = None,
        user_id: Optional[UUID] = None,
        duration: Optional[models.Int64] = None,
        category: Optional[str] = None,
        usage_data: Optional[dict | list] = None,
    ):
        return self._usage_record(
            timestamp=timestamp,
            datastore_id=datastore_id,
            user_id=user_id,
            duration=duration,
            category=category,
            usage_data=usage_data,
        )

    def _usage_record_by_model(self, data: models.UsageRecordCreateRequest):
        return self.usage_record(**data.model_dump())

    @abstractmethod
    def _user(self, **kwargs) -> models.UserDataResponse:
        pass

    def user(
        self,
        username: str,
        role_id: Optional[UUID] = None,
        admin: Optional[bool] = False,
        disabled: Optional[bool] = False,
        managed: Optional[bool] = False,
        external_id: Optional[str] = None,
        password: Optional[str] = None,
        context: Optional[dict] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        phone: Optional[str] = None,
        profile_picture: Optional[str] = None,
        company: Optional[str] = None,
        job_title: Optional[str] = None,
        language: Optional[str] = None,
        time_zone: Optional[str] = None,
        address_line_1: Optional[str] = None,
        address_line_2: Optional[str] = None,
        city: Optional[str] = None,
        state: Optional[str] = None,
        postal_code: Optional[str] = None,
        country: Optional[str] = None,
    ) -> models.UserDataResponse:
        return self._user(
            username=username,
            role_id=role_id,
            admin=admin,
            disabled=disabled,
            managed=managed,
            external_id=external_id,
            password=password,
            context=context,
            first_name=first_name,
            last_name=last_name,
            phone=phone,
            profile_picture=profile_picture,
            company=company,
            job_title=job_title,
            language=language,
            time_zone=time_zone,
            address_line_1=address_line_1,
            address_line_2=address_line_2,
            city=city,
            state=state,
            postal_code=postal_code,
            country=country,
        )

    def _user_by_model(self, data: models.UserCreateRequest) -> models.UserDataResponse:
        return self.user(**data.model_dump())



from typing import Optional

from pydantic import BaseModel

from lqs.interface.core.models.__common__ import (
    CommonModel,
    DataResponseModel,
    PaginationModel,
    optional_field,
)


class Workflow(CommonModel):
    name: str
    note: Optional[str]
    context: Optional[dict]
    managed: bool
    default: bool
    disabled: bool
    context_schema: Optional[dict]


class WorkflowDataResponse(DataResponseModel[Workflow]):
    pass


class WorkflowListResponse(PaginationModel[Workflow]):
    pass


class WorkflowCreateRequest(BaseModel):
    name: str
    note: Optional[str] = None
    context: Optional[dict] = None
    default: bool = False
    disabled: bool = False
    managed: bool = False
    context_schema: Optional[dict] = None


class WorkflowUpdateRequest(BaseModel):
    name: str = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    default: bool = optional_field
    disabled: bool = optional_field
    context_schema: Optional[dict] = optional_field



from lqs.interface.dsm import FetchInterface
from lqs.client.common import RESTInterface
import lqs.interface.dsm.models as models


class Fetch(FetchInterface, RESTInterface):
    service: str = "dsm"

    def __init__(self, app):
        super().__init__(app=app)

    def _announcement(self, **params):
        announcement_id = params.pop("announcement_id")
        result = self._get_resource(
            f"announcements/{announcement_id}",
            response_model=models.AnnouncementDataResponse,
        )
        return result

    def _api_key(self, **params):
        api_key_id = params.pop("api_key_id")
        result = self._get_resource(
            f"apiKeys/{api_key_id}", response_model=models.APIKeyDataResponse
        )
        return result

    def _comment(self, **params):
        comment_id = params.pop("comment_id")
        result = self._get_resource(
            f"comments/{comment_id}", response_model=models.CommentDataResponse
        )
        return result

    def _configuration(self, **params):
        configuration_id = params.pop("configuration_id")
        result = self._get_resource(
            f"configurations/{configuration_id}",
            response_model=models.ConfigurationDataResponse,
        )
        return result

    def _datastore(self, **params):
        datastore_id = params.pop("datastore_id")
        result = self._get_resource(
            f"dataStores/{datastore_id}", response_model=models.DataStoreDataResponse
        )
        return result

    def _datastore_association(self, **params):
        datastore_association_id = params.pop("datastore_association_id")
        result = self._get_resource(
            f"dataStoreAssociations/{datastore_association_id}",
            response_model=models.DataStoreAssociationDataResponse,
        )
        return result

    def _event(self, **params):
        event_id = params.pop("event_id")
        result = self._get_resource(
            f"events/{event_id}", response_model=models.EventDataResponse
        )
        return result

    def _job(self, **params):
        job_id = params.pop("job_id")
        result = self._get_resource(
            f"jobs/{job_id}", response_model=models.JobDataResponse
        )
        return result

    def _me(self, **params):
        result = self._get_resource("users/me", response_model=models.MeDataResponse)
        return result

    def _role(self, **params):
        role_id = params.pop("role_id")
        result = self._get_resource(
            f"roles/{role_id}", response_model=models.RoleDataResponse
        )
        return result

    def _user(self, **params):
        user_id = params.pop("user_id")
        result = self._get_resource(
            f"users/{user_id}", response_model=models.UserDataResponse
        )
        return result



from typing import List, Optional
from uuid import UUID

from pydantic import Field

from lqs.interface.core.models.__common__ import (
    CommonModel,
    DataResponseModel,
    LockModel,
    LockCreateRequest,
    LockUpdateRequest,
    PaginationModel,
    optional_field,
    Int64,
)

from lqs.interface.core.models import Topic


class Log(CommonModel, LockModel):
    group_id: UUID = Field(
        ..., description="The ID of the group to which this log belongs."
    )
    name: str = Field(..., description="The name of the log (unique per group).")

    start_time: Optional[Int64] = Field(
        ..., description="The timestamp of the first record of the log."
    )
    end_time: Optional[Int64] = Field(
        ..., description="The timestamp of the last record of the log."
    )
    record_size: int = Field(
        ..., description="The total size of all records in the log in bytes."
    )
    record_count: int = Field(
        ..., description="The total number of records in the log."
    )
    object_size: int = Field(
        ..., description="The total size of all objects in the log in bytes."
    )
    object_count: int = Field(
        ..., description="The total number of objects in the log."
    )

    note: Optional[str] = Field(
        ..., description="A general note about the log for reference."
    )
    context: Optional[dict] = Field(..., description="A JSON context for the log.")
    default_workflow_id: Optional[UUID] = Field(
        ...,
        description="The ID of the workflow to be executed during state transitions of associated processes.",
    )

    def list_topics(self, **kwargs) -> List["Topic"]:
        return self._app.list.topic(log_id=self.id, **kwargs).data


Log.model_rebuild()


class LogDataResponse(DataResponseModel[Log]):
    pass


class LogListResponse(PaginationModel[Log]):
    pass


class LogCreateRequest(LockCreateRequest):
    group_id: UUID
    name: str
    note: Optional[str] = None
    context: Optional[dict] = None
    default_workflow_id: Optional[UUID] = None


class LogUpdateRequest(LockUpdateRequest):
    group_id: UUID = optional_field
    name: str = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    default_workflow_id: Optional[UUID] = optional_field



from typing import List, Optional
from uuid import UUID

from lqs.interface.core.models.__common__ import (
    CommonModel,
    DataResponseModel,
    LockModel,
    LockCreateRequest,
    LockUpdateRequest,
    PaginationModel,
    optional_field,
)
from lqs.interface.core.models import Log


class Group(CommonModel, LockModel):
    name: str
    note: Optional[str]
    context: Optional[dict]
    default_workflow_id: Optional[UUID]

    def list_logs(self, **kwargs) -> List["Log"]:
        return self._app.list.logs(group_id=self.id, **kwargs).data


class GroupDataResponse(DataResponseModel[Group]):
    pass


class GroupListResponse(PaginationModel[Group]):
    pass


class GroupCreateRequest(LockCreateRequest):
    name: str
    note: Optional[str] = None
    context: Optional[dict] = None
    default_workflow_id: Optional[UUID] = None


class GroupUpdateRequest(LockUpdateRequest):
    name: str = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    default_workflow_id: Optional[UUID] = optional_field



from typing import Optional
from uuid import UUID

from pydantic import Field

from lqs.interface.core.models.__common__ import (
    DataResponseModel,
    PaginationModel,
    ProcessModel,
    ProcessCreateRequest,
    ProcessUpdateRequest,
    optional_field,
)


class Ingestion(ProcessModel["Ingestion"]):
    log_id: UUID = Field(
        ..., description="The ID of the log to which this ingestion belongs."
    )

    object_store_id: Optional[UUID] = Field(
        ...,
        description="If the ingestion object is stored in an object store, the ID of the object store.",
    )
    object_key: Optional[str] = Field(
        ..., description="The key of the ingestion object."
    )

    name: Optional[str] = Field(
        ..., description="The name of the ingestion (not unique)."
    )
    note: Optional[str] = Field(
        ..., description="A general note about the ingestion for reference."
    )
    context: Optional[dict] = Field(
        ..., description="A JSON context for the ingestion."
    )

    group_id: Optional[UUID] = Field(
        ...,
        description="The ID of the group to which this ingestion belongs.",
    )


class IngestionDataResponse(DataResponseModel[Ingestion]):
    pass


class IngestionListResponse(PaginationModel[Ingestion]):
    pass


class IngestionCreateRequest(ProcessCreateRequest):
    log_id: UUID
    name: Optional[str] = None
    object_store_id: Optional[UUID] = None
    object_key: Optional[str] = None
    note: Optional[str] = None
    context: Optional[dict] = None


class IngestionUpdateRequest(ProcessUpdateRequest):
    name: Optional[str] = optional_field
    object_store_id: Optional[UUID] = optional_field
    object_key: Optional[str] = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field



from lqs.interface.dsm.create import CreateInterface
from lqs.interface.dsm.delete import DeleteInterface
from lqs.interface.dsm.fetch import FetchInterface
from lqs.interface.dsm.list import ListInterface
from lqs.interface.dsm.update import UpdateInterface
